import org.w3c.dom.*;

import javax.xml.parsers.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;

import java.sql.*;

/**
 * @author <a href="mailto:fpb@di.fct.unl.pt">Fernando Birra</a>
 * <p>
 * This program shows how to populate a simple database with spatial information.
 * The data comes from an XML file containing the <b>wkt</b> representation of the spatial data.
 */



public class Main {

	
	static private Connection connect = null;
	static private PreparedStatement preparedStatement = null;
	

	/**
	 * Opens  the connection to the databse. You must have the MySQL JDBC driver installed.
	 * @throws ClassNotFoundException when the mysql JDBC driver isn't found.
	 * @throws SQLException if the connection can't be made after loading the driver.
	 */
	public static void openDB() throws ClassNotFoundException, SQLException   {
		// This will load the MySQL driver, each DB has its own driver
		Class.forName("com.mysql.jdbc.Driver");
		// Setup the connection with the DB
		connect = DriverManager.getConnection("jdbc:mysql://localhost/campus?"
						+ "user=someuser&password=somepwd");		
	}
	
	/**
	 * Closes the connection with the database.
	 */
	public static void closeDB()
	{
		try {
			if (connect != null) {
				connect.close();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}		
	}
	
	/**
	 * Inserts data into a relational database table. The query is previously prepared
	 * and just the actual values are inserted before executing it.
	 * @param id identifier of the geometric feature
	 * @param desc description of the geometric feature
	 * @param g geometry information about the feature to be inserted in wkt format
	 *  (actual geographic information, like Point, Polygon, MultiPolygon, etc)
	 */
	public static void insertGeoData(String id, String desc, String g) {
		try {
			if(preparedStatement==null) // The statement hasn't been prepared yet...
				preparedStatement = connect.prepareStatement("insert into spaces values (?, ?, GeomFromText(?))");
			preparedStatement.setString(1, id);
			preparedStatement.setString(2, desc);
			preparedStatement.setString(3, g);
			preparedStatement.executeUpdate();
		} catch (Exception e) {
			System.out.println(e.toString());
		} finally {
//			close();
		}	
	}
	
	

	/**
	 * This method parses the description element and collects the needed attributes inside a Map.
	 * There are other attributes present in the input file. We only collect the ones we need to
	 * populate the table columns.
	 * @param n Node corresponding to the <description> tag
	 * @return Map<String,String> that translates attribute names into their corresponding values
	 */
	static Map<String,String> parseDescription(Node n)
	{
		Map<String,String> m = new HashMap<String,String>();
		
		Element e = (Element) n;
		NodeList meta = e.getElementsByTagName("meta-information");
		// The meta-information element is supposed to be unique for each placemark.
		// Nevertheless we are prepared to handle several instances of it.
		for(int i=0; i<meta.getLength(); i++) {
			Node entry = meta.item(i);
			String k = entry.getAttributes().getNamedItem("attribute").getTextContent();
			String v = entry.getAttributes().getNamedItem("value").getTextContent();
			m.put(k, v);
		}
		return m;
	}
		
	/**
	 * Collects the information present in the description node as well as the Geographic information inside the geo-wkt node
	 * and inserts it in the database. The description fileds used are: ID and Nome.
	 * @param n Node corresponding to the <Placemark> tag
	 */
	static void parsePlacemark(Node n) 
	{
		Element e = (Element) n;
		
		NodeList descrs = e.getElementsByTagName("description");
		Map<String,String> m = parseDescription(descrs.item(0));
		
		// This is the ugly part. We need to investigate which type of placemark we're dealing with...
		System.out.print("("+m.get("ID") + ")");

		NodeList wkts = e.getElementsByTagName("geo-wkt");
		System.out.println(wkts.item(0).getTextContent());
		
		insertGeoData(m.get("ID"), m.get("Nome"), wkts.item(0).getTextContent());
		
	}
	
	/**
	 * Processes a list of Placemarks and for each one, parses it and inserts the corresponding information inside the database table.
	 * @param nodes List of Nodes corresponding to Placemarks inside the input XML file.
	 */
	static void parsePlacemarks(NodeList nodes) 
	{
		for(int i=0; i<nodes.getLength(); i++)
			parsePlacemark(nodes.item(i));
	}
	
	/**
	 * The function that actually loads the file, builds the DOM object and
	 * opens the connection to the DB. After that each placemark is handled in turn.
	 * 
	 * @param filename Name of the file to be processed.
	 * @throws Exception 
	 */
	public static void parseFile(String filename) throws Exception
	{
        DocumentBuilderFactory fact = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = fact.newDocumentBuilder();
        Document doc = builder.parse(filename);
        Node node = doc.getDocumentElement();
        String root = node.getNodeName();
        System.out.println("Root Node: " + root);
        
        openDB();
        
        NodeList places = doc.getElementsByTagName("Placemark");
        System.out.println("File contains: " + places.getLength() + " places");
       
        parsePlacemarks(places);

        closeDB();
	}
			
	/**
	 * This program should be invoked with the filename that stores the spatial data. 
	 * If the connection with the database isn't working check that: the JDBC driver is
	 * installed and in the classpath. Also adjust the database connection parameters in
	 * @see #openDB()
	 * @param args commandline args. There should be at least one argument - the file to processed.
	 * @throws Exception 
	 */
	public static void main(String[] args) {
		try{
			String str = args[0];
		    File file = new File(str);
		    if (file.exists()){
		    	parseFile(str);
		    }
		    else{
		    	System.out.println("File not found!");
		    }
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}	
}